#[allow(non_snake_case)]

use std::{thread, time};
use std::sync::Arc;
use crate::mpsc::Sender;
use std::ffi::CString;
use std::ffi::CStr;

#[link(name = "mxnet2api")]
unsafe extern "C"{ 
    unsafe fn rInit_MatrixAPI()->i16; 
    unsafe fn rRelease_MatrixAPI()->i16;
    unsafe fn rDongle_Count(DngPort:u16)->i16;
    unsafe fn rDongle_ReadSerNr( UserCode:i32, DngNr:i16, DngPort:i16)->i32;
    unsafe fn rLogIn_MatrixNet( UserCode:i32, appSlot:i16, DngNr:i16, DngPort:i16)->i16;
#[cfg(target_os = "windows")]
    unsafe fn MxNet_SetIPA(ip: *const i8)->*const i8;
#[cfg(any(target_os = "macos",target_os = "linux"))]
    unsafe fn MxNet_SetIP(ip: *const u8)->*const u8;
    unsafe fn MxNet_SetPort(port: i32);
    #[allow(dead_code)]
    unsafe fn rLogOut_MatrixNet( UserCode:i32, appSlot:i16, DngNr:i16, DngPort:i16)->i16;
}


pub struct Api{
    user_code:i32,
}

impl Api{
    pub fn new( uc: i32) -> Self{
        Self
         { 
            user_code : uc,
         }
    }
   pub fn init(&self) -> i16{
       unsafe{ rInit_MatrixAPI() }
    }

   pub fn release(&self) -> i16{
       unsafe{ rRelease_MatrixAPI() }
    }

    pub fn count(&self) -> i16{
       unsafe{ rDongle_Count(85) }
    }

    pub fn sernr(&self) -> i32{
       unsafe{ rDongle_ReadSerNr(self.user_code, 1,85) }
    }

    pub fn login(&self, app_slot:i16) -> i16{
       unsafe{ rLogIn_MatrixNet(self.user_code, app_slot, 1,85) }
    }

#[cfg(target_os = "windows" )]
    #[allow(dead_code)]
    pub fn set_ip(&self, ip:&str)->bool
    {
       let c_ip = CString::new(ip).unwrap();
       let setip = unsafe{
            let c_setip = MxNet_SetIPA(c_ip.as_ptr());
            CStr::from_ptr(c_setip).to_str().unwrap()
        };
//      println!("setip {setip}");
        if setip=="" { false } else {true}
	}

#[cfg(any(target_os = "macos",target_os="linux") )]
    #[allow(dead_code)]
    pub fn set_ip(&self, ip:&str)->bool
    {
       let c_ip = CString::new(ip).unwrap();
       let setip = unsafe{
            let c_setip = MxNet_SetIP(c_ip.as_ptr() as *const u8);
            CStr::from_ptr(c_setip as *const i8).to_str().unwrap()
        };
//      println!("setip {setip}");
        if setip=="" { false } else {true}
	}

    #[allow(dead_code)]
    pub fn set_port(&self, port:i32)
    {
       unsafe{
            MxNet_SetPort(port);
        }
    }

    #[allow(dead_code)]
    pub fn logout(&self, app_slot:i16) -> i16{
       unsafe{ rLogOut_MatrixNet(self.user_code, app_slot, 1,85) }
    }
}



struct LicenseInner{
    mxnet2 : Api,
    appslot : i16,
    interval: u64,
}

#[derive(Clone)]
pub struct License{
    inner: Arc<LicenseInner>
}

impl Drop for License{
    fn drop(&mut self)
    {
        println!("release");
        self.inner.mxnet2.release();
    }
}


impl License{

    pub fn new( uc:i32, slot:i16, interval_second: u64) ->Self{
        Self {
             inner: Arc::new( LicenseInner { mxnet2: Api::new(uc), appslot: slot, interval:interval_second })
         }
    }

    #[allow(dead_code)]
    pub fn set_port(&self, port:i32) {
        self.inner.mxnet2.set_port(port);
    }

    #[allow(dead_code)]
    pub fn set_ip(&self, ip:&str)->bool {
        self.inner.mxnet2.set_ip(ip)
    }

    pub fn acquire(&self) -> bool{

        let local_self = self.inner.clone();

        let handle = thread::spawn(move || {

            let status = local_self.mxnet2.init();
            if status < 0{
                println!("Init error");
                return false;
            }

            if local_self.mxnet2.count() <=0 {
                println!("Network dongle not found");
                local_self.mxnet2.release();
                return false;
            }

            let sernr =  local_self.mxnet2.sernr();
            println!("SerNr {sernr}" );
            if sernr <= 0 {
                local_self.mxnet2.release();
                return false;
            }

            let login=local_self.mxnet2.login(local_self.appslot);
            println!("login: {login}" );
            if login < 0{
                local_self.mxnet2.release();
                return false
            }

            true
        });

        println!("Waiting for license acqusition");

        let  res = handle.join();
        match res{
            Ok(status) => { if status {true} else {false} },
            Err(_msg) => false,
        }

    }


    pub fn retain(&self, tx:Sender<i16>) {
        
        let local_self = self.inner.clone();
        let mut retry100=0;

        thread::spawn(move || {

            loop{
                let interval_second = time::Duration::from_secs(local_self.interval);
                thread::sleep(interval_second);

                let login=local_self.mxnet2.login(local_self.appslot);
                println!("login: {login}" );
                if login < 0{
                    if login == -105
                    {
                        if local_self.mxnet2.init() >=0 {continue;}
                    }
                    else if login == -100
                    {
                        retry100 +=1;
                        if retry100 < 3 { continue; }
                    }
                    tx.send(4).unwrap();
                }
            }
        });

    }

    pub fn release(&self)
    {
        self.inner.mxnet2.release();
    }
}

